@@ -7,13 +7,16 @@
 
 #include "Mutate.h"
 #include <fstream>
+#include "llvm/Support/CommandLine.h"
 
 using namespace llvm;
 
-cl::opt<std::string> Inst1("inst1", cl::init("0"), cl::desc("first statement to mutate"));
+// cl::opt<std::string> Inst1("inst1", cl::init("0"), cl::desc("first statement to mutate"));
+static cl::list<std::string, bool> InstList1("inst1", cl::desc("first statement to mutate"), cl::ZeroOrMore);
 cl::opt<std::string> Inst1ID("inst1ID", cl::init(""), cl::desc("The Unique ID of first statement to mutate"));
 
-cl::opt<std::string> Inst2("inst2", cl::init("0"), cl::desc("second statement to mutate"));
+// cl::opt<std::string> Inst2("inst2", cl::init("0"), cl::desc("second statement to mutate"));
+static cl::list<std::string, bool> InstList2("inst2", cl::desc("second statement to mutate"), cl::ZeroOrMore);
 cl::opt<std::string> Inst2ID("inst2ID", cl::init(""), cl::desc("The Unique ID of first statement to mutate"));
 
 cl::opt<bool> use_result("use_result", cl::init(1), cl::desc("Whether to perform the use_result after changing the instruction"));
@@ -150,12 +153,14 @@ namespace {
   };
 }
 
+// We don't stack query operation with others, so no need to pop the insturction list
 namespace {
   struct Query : public ModulePass {
     static char ID;
     Query() : ModulePass(ID) {}
 
-    bool runOnModule(Module &M){
+    bool runOnModule(Module &M){;
+      std::string Inst1 = InstList1[0];
       Instruction *I = dyn_cast_or_null<Instruction>(walkExact(Inst1, Inst1ID, M, NULL, false));
       if (I == NULL) {
         errs() << "Query failed. Cannot find " << Inst1 << "\n";
@@ -191,9 +196,12 @@ namespace {
     Cut() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      std::string Inst1 = InstList1[0];
+      InstList1.erase(InstList1.begin());
+      InstList2.erase(InstList2.begin());
       Instruction *I = dyn_cast_or_null<Instruction>(walkExact(Inst1, Inst1ID, M, NULL, true));
       if (I == NULL) {
-        errs() << "cut failed. Cannot find/use " << Inst1 << "\n";
+        errs() << "cut failed("<< Inst1 << "). Cannot find/use " << Inst1 << "\n";
         return RET_ERROR; }
 
       insertNOP(I);
@@ -219,10 +227,14 @@ namespace {
 
     bool runOnModule(Module &M){
       Instruction *temp;
+      std::string Inst1 = InstList1[0];
+      std::string Inst2 = InstList2[0];
+      InstList1.erase(InstList1.begin());
+      InstList2.erase(InstList2.begin());
       Instruction *SI = walkCollect(Inst2, Inst2ID, M);
       Instruction *DI = walkPosition(Inst1, Inst1ID, M);
       if (SI == NULL or DI == NULL) {
-        errs()<<"insertion failed. Cannot find/use ";
+        errs()<<"insertion failed("<< Inst1 << "," << Inst2 << "). Cannot find/use ";
         if (DI == NULL) errs()<<Inst1 << " ";
         if (SI == NULL) errs()<<Inst2 << " ";
         errs() << "\n";
@@ -256,17 +268,22 @@ namespace {
 
     bool runOnModule(Module &M){
       Instruction *temp;
+      std::string Inst1 = InstList1[0];
+      std::string Inst2 = InstList2[0];
+      InstList1.erase(InstList1.begin());
+      InstList2.erase(InstList2.begin());
       Instruction *SI = walkCollect(Inst2, Inst2ID, M);
       Instruction *DI = walkPosition(Inst1, Inst1ID, M);
       if (SI == NULL or DI == NULL) {
-        errs()<<"replace failed. Cannot find/use ";
+        errs()<<"replace failed("<< Inst1 << "," << Inst2 << "). Cannot find/use ";
         if (DI == NULL) errs()<<Inst1 << " ";
         if (SI == NULL) errs()<<Inst2 << " ";
         errs() << "\n";
+        
         return RET_ERROR; }
 
       if (DI->getType() != SI->getType()) {
-        errs()<<"replace failed. could find no use for the result.\n";
+        errs()<<"replace failed("<< Inst1 << "," << Inst2 << "). could find no use for the result.\n";
         return RET_ERROR;
       }
 
@@ -290,11 +307,15 @@ namespace {
 
     bool runOnModule(Module &M){
       int err = 0;
+      std::string Inst1 = InstList1[0];
+      std::string Inst2 = InstList2[0];
+      InstList1.erase(InstList1.begin());
+      InstList2.erase(InstList2.begin());
       // Type* T;
       if (Inst1 == "Rand" && Inst2 != "Rand") {
         Value* sval = walkExact(Inst2, Inst2ID, M, NULL, false);
         if (sval == NULL){
-          errs() << "oprepl failed. cannot find " << Inst2 << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find " << Inst2 << "\n";
           return RET_ERROR;
         }
         std::pair<Instruction*, unsigned> result;
@@ -311,7 +332,7 @@ namespace {
             sval->getType()
           );
         if (result.first == NULL) {
-          errs() << "oprepl failed. cannot find an OP that has the same type as " << Inst2 << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find an OP that has the same type as " << Inst2 << "\n";
           return RET_ERROR;
         }
 
@@ -330,7 +351,7 @@ namespace {
         unsigned OPidx = std::stoi(dstOP.drop_front(2).str());// remove "OP"
         Instruction *DI = dyn_cast_or_null<Instruction>(walkExact(dstInstBase.str(), Inst1ID, M, NULL, false));
         if (DI == NULL){
-          errs() << "oprepl failed. cannot find" << dstInstBase << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find" << dstInstBase << "\n";
           return RET_ERROR;
         }
         Inst1ID = Inst1ID + ".OP" + std::to_string(OPidx);
@@ -360,7 +381,7 @@ namespace {
             sval->getType()
           );
         if (result.first == NULL) {
-          errs() << "oprepl failed. cannot find an OP that has the same type as " << Inst2ID << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find an OP that has the same type as " << Inst2ID << "\n";
           return RET_ERROR;
         }
 
@@ -381,15 +402,19 @@ namespace {
           return RET_WARNING;
         }
         else if (err == -1)
-          errs() << "oprepl failed. cannot find " << Inst1 << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find " << Inst1 << "\n";
         else if (err == -2)
-          errs() << "oprepl failed. out of operand index range" << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). out of operand index range" << "\n";
         else if (err == -3)
-          errs() << "oprepl failed. source instruction does not have an output" << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). source instruction does not have an output" << "\n";
         else if (err == -4)
-          errs() << "oprepl failed. type mismatched" << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). type mismatched" << "\n";
         else if (err == -5)
-          errs() << "oprepl failed. cannot find " << Inst2 << "\n";
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). cannot find " << Inst2 << "\n";
+        else if (err == -6)
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). dest operand " << Inst1 << " is a function argument with immarg attribute\n";
+        else if (err == -7)
+          errs() << "oprepl failed("<< Inst1 << "," << Inst2 << "). "<< Inst2 << " does not dominate " << Inst1 << "\n";
         return RET_ERROR;
       }
     }
@@ -402,12 +427,16 @@ namespace {
     Move() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      std::string Inst1 = InstList1[0];
+      std::string Inst2 = InstList2[0];
+      InstList1.erase(InstList1.begin());
+      InstList2.erase(InstList2.begin());
       Instruction *temp;
       Instruction *SI = dyn_cast_or_null<Instruction>(walkExact(Inst2, Inst2ID, M, NULL, true));
       Instruction *DI = walkPosition(Inst1, Inst1ID, M);
 
       if (SI == NULL or DI == NULL) {
-        errs()<<"Move failed. Cannot find/use ";
+        errs()<<"Move failed("<< Inst1 << "," << Inst2 << "). Cannot find/use ";
         if (DI == NULL) errs()<<Inst1 << " ";
         if (SI == NULL) errs()<<Inst2 << " ";
         errs() << "\n";
@@ -447,11 +476,15 @@ namespace {
     Swap() : ModulePass(ID) {}
 
     bool runOnModule(Module &M){
+      std::string Inst1 = InstList1[0];
+      std::string Inst2 = InstList2[0];
+      InstList1.erase(InstList1.begin());
+      InstList2.erase(InstList2.begin());
       Instruction *temp1, *temp2;
       Instruction *I1 = dyn_cast_or_null<Instruction>(walkExact(Inst1, Inst1ID, M, NULL, true));
       Instruction *I2 = dyn_cast_or_null<Instruction>(walkExact(Inst2, Inst2ID, M, NULL, true));
       if (I1 == NULL or I2 == NULL) {
-        errs()<<"swap failed. Cannot find/use ";
+        errs()<<"swap failed("<< Inst1 << "," << Inst2 << "). Cannot find/use ";
         if (I1 == NULL) errs()<<Inst1 << " ";
         if (I2 == NULL) errs()<<Inst2 << " ";
         errs() << "\n";
@@ -497,6 +530,10 @@ namespace {
     static char ID; // pass identification
     Cache() : ModulePass(ID) {}
     bool runOnModule(Module &M){
+      std::string Inst1 = InstList1[0];
+      std::string Inst2 = InstList2[0];
+      InstList1.erase(InstList1.begin());
+      InstList2.erase(InstList2.begin());
       Instruction *I;
       if (Inst1 == "Rand") {
         std::pair<Instruction*, StringRef> result = randTexCachableI(M);
@@ -506,7 +543,7 @@ namespace {
       else {
         I = dyn_cast_or_null<Instruction>(walkExact(Inst1, Inst1ID, M, NULL, true));
         if (I == NULL) {
-          errs() << "cache failed. Cannot find/use " << Inst1 << "\n";
+          errs() << "cache failed("<< Inst1 << "," << Inst2 << "). Cannot find/use " << Inst1 << "\n";
           return RET_ERROR; }
       }
 
