@@ -11,10 +11,12 @@ RET_WARNING = 1
 RET_SUCCESS = 0
 
 try:
+    import ast
     import gevo._llvm
     from gevo import __version__
     from gevo._llvm import __llvm_version__
     from gevo._llvm import __llvm_bin_folder__
+    from gevo import irind
 
 except ImportError:
     # TODO: load mutation.so from Mutate folder with proper llvm version string as a standalone mode
@@ -54,11 +56,25 @@ if __name__ == '__main__':
             previous_act.append((self.dest, values))
             setattr(namespace, 'mutation_commands', previous_act)
 
+    class LoadFromFile (argparse.Action):
+        def __call__ (self, parser, namespace, values, option_string = None):
+            with open(values, 'r') as f:
+                editListRaw = ast.literal_eval(f.read())
+                editList = irind.encode_edits_from_list(editListRaw)
+                serialized_edits = irind.decode_edits(editList, 'edit')
+                serialized_edits = irind.sort_serialized_edits(serialized_edits)
+                editStr = ' '.join(irind.decode_edits(editList, 'str'))
+                # parse arguments in the file and store them in the target namespace
+                _args = parser.parse_args(editStr.split(), namespace)
+                print(f'edit file contains {len(serialized_edits)} edits, {len(_args.mutation_commands)} operations')
+
     llvm_mutate_parser = argparse.ArgumentParser(description="Manipulate LLVM assembly from Nvidia CUDA Kernels")
     llvm_mutate_parser.add_argument('-o', '--output_file', metavar='FILE', type=FileOpener('w'), default='-',
         help='output file name. Output will be redirected to stdout if output file is not specified')
     llvm_mutate_parser.add_argument('-f', '--input_file', metavar='FILE', type=FileOpener('r'), default='-',
         help='input file name. llvm-mutate will use and wait for stdin as the input stream if input file is not specified.')
+    llvm_mutate_parser.add_argument('-ef', '--edit_file', metavar='FILE', action=LoadFromFile,
+        help='edit file name to feed edits instead of using commands. edit file contain edits under gevo format.')
     llvm_mutate_parser.add_argument('-n', '--name', action='store_true',
         help='add unique ID (UID) for each instruction. UID is one of needed instruction description in mutation operations')
     llvm_mutate_parser.add_argument('-I', '--ids', action='store_true',
@@ -79,7 +95,7 @@ if __name__ == '__main__':
                      integer number as instruction index or Unique ID. The advantage of UID is,  \
                      once created, persistence throughout the mutation operation as long as the \
                      the instruction UID denoted is not the target of mutation operation. \
-                     Note: --name, --ids, and mutation operation cannot be used together.')
+                     Note: --name, --ids, --edit_file and mutation operation cannot be used together.')
     mutation_operation_group.add_argument(
         '-c', '--cut', type=str, dest='-cut', action=MutationAction, metavar='INST',
         help='cut the given instruction')
@@ -103,10 +119,13 @@ if __name__ == '__main__':
     ret_code = RET_SUCCESS
     OPT_NOT_USE_RESULT = '-use_result=0' if args.not_use_result else ''
 
+    if 'mutation_commands' and args.edit_file in args:
+        print("Warning: edit file will overwrite the mutation operations!")
+
     if args.name and 'mutation_commands' in args:
         print("--name and mutation operations cannot be used together.")
         sys.exit(RET_ERROR)
-
+    
     if args.name:
         opt_proc = subprocess.Popen(
             [ opt_path,
@@ -139,7 +158,7 @@ if __name__ == '__main__':
         opt_proc = subprocess.Popen(
             [ opt_path,
               '-load', LLVM_MUTATE_LIBRARY_PATH,
-              '-query', f'-inst1={args.query}'],
+              '-query', f'-inst1={args.query}', '-inst2='],
             stdin=args.input_file.open(),
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE )
@@ -153,40 +172,27 @@ if __name__ == '__main__':
     elif 'mutation_commands' in args:
         with args.input_file.open() as f:
             input_str = f.buffer.read()
+
+        opt_args = [opt_path, '-load', LLVM_MUTATE_LIBRARY_PATH,
+                    '-use_result=0'] if args.not_use_result else\
+                   [opt_path,'-load', LLVM_MUTATE_LIBRARY_PATH, ]
         for mop in args.mutation_commands:
             insts = mop[1].split(',')
-            inst_args = [ f'-inst1={insts[0]}' ] if len(insts) == 1 else\
-                        [ f'-inst1={insts[0]}', f'-inst2={insts[1]}']
-            opt_args = [opt_path,
-                        '-load', LLVM_MUTATE_LIBRARY_PATH,
-                        '-use_result=0', mop[0]] if args.not_use_result else\
-                       [opt_path,
-                        '-load', LLVM_MUTATE_LIBRARY_PATH, mop[0]]
-
+            inst_args = [ mop[0], f'-inst1={insts[0]}', '-inst2=' ] if len(insts) == 1 else\
+                        [ mop[0], f'-inst1={insts[0]}', f'-inst2={insts[1]}']
             opt_args.extend(inst_args)
 
-            opt_proc = subprocess.Popen(
-                opt_args,
-                stdin=subprocess.PIPE,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE )
-            opt_stdout, opt_stderr = opt_proc.communicate(input=input_str)
+        opt_proc = subprocess.Popen(
+            opt_args,
+            stdin=subprocess.PIPE,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE )
+        opt_stdout, opt_stderr = opt_proc.communicate(input=input_str)
 
-            if opt_proc.returncode != 0:
-                print(f"llvm-mutate: opt error in {mop[0]} {mop[1]}")
-                print(opt_stderr.decode(), end='')
-                sys.exit(opt_proc.returncode)
-
-            # TODO: Have proper return code from Mutate.so. irind.py in gevo need to be changed as well
-            print(opt_stderr.decode(), end='', file=sys.stderr)
-            if args.interrupt_at_fail:
-                if opt_stderr.decode().find('failed') != -1:
-                    sys.exit(RET_SUCCESS)
-                if opt_stderr.decode().find('mismatch') != -1:
-                    sys.exit(RET_SUCCESS)
-                if opt_stderr.decode().find('no use') != -1:
-                    sys.exit(RET_SUCCESS)
-            input_str = opt_stdout
+        if opt_proc.returncode != 0:
+            print(opt_stderr.decode(), end='')
+            sys.exit(opt_proc.returncode)
+        print(opt_stderr.decode(), end='', file=sys.stderr)
     else:
         llvm_mutate_parser.print_help()
         sys.exit(RET_ERROR)
